# Package: PENTATrainer2
# Version: 1.1
# Filename: learn.praat
#
# Author: Santitham Prom-on, Yi Xu
# Last updated: 4 July 2016
#
# Objective: learning underlying target using stochastic optimization
# (e.g. simulated annealing) and analysis-by-synthesis

# Select the working directory
directory$ = chooseDirectory$ ("Choose a directory to optimize")
if directory$ = ""
	#[EXIT_00] Choose to not Continue
	exit
else
	#Check validity of the directory
	Create Strings as file list... FileList 'directory$'/*.wav
	numberOfFiles = Get number of strings
	if !numberOfFiles
		select Strings FileList
		Remove
		Create Strings as file list... FileList 'directory$'/*.WAV
		numberOfFiles = Get number of strings
	endif
	
	#[EXIT_L01] No sound files in the directory	
	if numberOfFiles = 0
		exit Error Code L01: No sound files in the directory
	else
		Write to raw text file... 'directory$'/FileList.txt
	endif
endif

### Error Checking ###

#[EXIT_L02] Cannot find configuration file: config.txt
if not fileReadable("'directory$'/config.txt")
	exit Error Code L02: Cannot find configuration file: config.txt
endif

#[EXIT_L03] Cannot find function profile file: function_profile.txt
if not fileReadable("'directory$'/function_profile.txt")
	exit Error Code L03: Cannot find function profile file: function_profile.txt
endif

Create Strings as file list... aList 'directory$'/*.annotation
n_a = Get number of strings
if n_a = 0
	#[EXIT_L04] Cannot find annotation files
	exit Error Code L04: Cannot find annotation files	
elsif n_a < numberOfFiles
	#[EXIT_L05] Number of annotation files must equal number of sound files
	exit Error Code L05: Number of annotation files must equal number of sound files
endif

for i from 1 to n_a
	select Strings aList
	fileName$ = Get string... i
	name$ = fileName$ - ".annotation"
	nocheck Read from file... 'directory$'/'fileName$'
	fullName$ = selected$()
	name1$ = extractLine$(fullName$," ")
	if (name$ = name1$)
		Remove
	else
		#[EXIT_L06] Wrong format (annotation)
		exit Error Code L06: Wrong format ('name$'.annotation)
	endif
endfor
select Strings aList
Remove

Create Strings as file list... iList 'directory$'/*.intervals
n_i = Get number of strings
if n_i = 0
	#[EXIT_L07] Cannot find intervals files
	exit Error Code L07: Cannot find intervals files	
elsif n_i < numberOfFiles
	#[EXIT_L08] Number of intervals files must equal number of sound files
	exit Error Code L08: Number of intervals files must equal number of sound files
endif

for i from 1 to n_i
	select Strings iList
	fileName$ = Get string... i
	name$ = fileName$ - ".intervals"
	nocheck Read from file... 'directory$'/'fileName$'
	fullName$ = selected$()
	name1$ = extractLine$(fullName$," ")
	if (name$ = name1$)
		Remove
	else
		#[EXIT_L09] Wrong format (intervals)
		exit Error Code L06: Wrong format ('name$'.intervals)
	endif
endfor
select Strings iList
Remove

Create Strings as file list... fList 'directory$'/*.actutimenormf0
n_f = Get number of strings
if n_f = 0
	#[EXIT_L10] Cannot find actutimenormf0 files
	exit Error Code L10: Cannot find actutimenormf0 files	
elsif n_f < numberOfFiles
	#[EXIT_L11] Number of actutimenormf0 files must equal number of sound files
	exit Error Code L11: Number of actutimenormf0 files must equal number of sound files
endif
for i from 1 to n_f
	select Strings fList
	fileName$ = Get string... i
	name$ = fileName$ - ".actutimenormf0"
	nocheck Read from file... 'directory$'/'fileName$'
	fullName$ = selected$()
	name1$ = extractLine$(fullName$," ")
	if (name$ = name1$)
		Remove
	else
		#[EXIT_L12] Wrong format (actutimenormf0)
		exit Error Code L12: Wrong format ('name$'.actutimenormf0)
	endif
endfor
select Strings fList
Remove

Read from file... 'directory$'/config.txt
c_nrows = Get number of rows
if c_nrows <> 7
	#[EXIT_L13] Wrong format (config.txt)
	exit Error Code L13: Wrong format (config.txt)
endif
Remove

Read from file... 'directory$'/function_profile.txt
c_nrows = Get number of rows
if c_nrows <> 3
	#[EXIT_L13] Wrong format (config.txt)
	exit Error Code L13: Wrong format (config.txt)
endif
Remove

### End of Error Checking ###

beginPause("Learn - Options")
	comment("Configure optimization parameters")
	integer("Maximum Iteration",700)
	real("Learning Rate",0.1)
	real("Starting Temperature", 700)
	real("Reduction Factor", 0.97)
	real("Silent Threshold",0.20)
	boolean("Inspect Manipulation",0)
clicked = endPause("Cancel","Start",2,1)

#[EXIT_00] Choose to not Continue
if clicked = 1
	exit
endif


Create Strings as file list... IntervalList 'directory$'/*.intervals
nintervals = Get number of strings
Remove
if !nintervals
	Create Strings as file list... AnnotationList 'directory$'/*.annotation
	n_annot = Get number of strings
	for i from 1 to n_annot
		Read from file... 'directory$'/'name$'.annotation
		Down to Table... no 4 yes no
		Save as tab-separated file... 'directory$'/'name$'.intervals
		select TextGrid 'name$'
		plus Table 'name$'
		Remove
	endfor
	select Strings AnnotationList
	Remove
endif
	
system java -jar Learn.jar "'directory$'" 'maximum_Iteration' 'learning_Rate' 'starting_Temperature' 'reduction_Factor' 'silent_Threshold'


select Strings FileList
numberOfFiles = Get number of strings
Create Table with column names... accuracy numberOfFiles+1 Filename RMSE Correlation
mean_rmse = 0
mean_corr = 0
for current_file from 1 to numberOfFiles
	select Strings FileList
	fileName$ = Get string... current_file
	name$ = fileName$ - ".wav" - ".WAV"
	
	Read from file... 'directory$'/'name$'.intervals
	start_time = Get value... 1 tmin
	select Table 'name$'
	Remove
	
	Read from file... 'directory$'/'name$'.synf0
	nrow = Get number of rows
	rel_end_time = Get value... nrow Time
	end_time = rel_end_time + start_time
	Create PitchTier... 'name$' start_time end_time
	
	sxy = 0
	sx = 0
	sy = 0
	sx2 = 0
	sy2 = 0
	sse = 0
	for i from 1 to nrow
		select Table 'name$'
		t_rel = Get value... i Time
		t = t_rel + start_time
		f0 = Get value... i F0syn
		orgf0 = Get value... i F0org
		f0hz = exp((1/12)*f0*ln(2))
		select PitchTier 'name$'
		Add point... 't' 'f0hz'
		
		sxy = sxy + f0*orgf0
		sx = sx + f0
		sy = sy + orgf0
		sx2 = sx2 + f0*f0
		sy2 = sy2 + orgf0*orgf0
		
		sse = sse + (f0-orgf0)*(f0-orgf0)
	endfor
	
	rmse = sqrt(sse/nrow)
	corr = (nrow*sxy - sx*sy)/(sqrt(nrow*sx2-sx*sx)*sqrt(nrow*sy2-sy*sy))
	mean_rmse += rmse
	mean_corr += corr

	select Table accuracy
	Set string value... current_file Filename 'name$'
	Set numeric value... current_file RMSE 'rmse'
	Set numeric value... current_file Correlation 'corr'
	
	if inspect_Manipulation
		Read from file... 'directory$'/'fileName$'
		To Manipulation... 0.01 75 500
		plus PitchTier 'name$'
		Replace pitch tier
	endif
	
	select Table 'name$'
	plus PitchTier 'name$'
	if inspect_Manipulation
		plus Sound 'name$'
	endif
	Remove
endfor
mean_rmse /= numberOfFiles
mean_corr /= numberOfFiles

select Table accuracy
Set string value... numberOfFiles+1 Filename Mean
Set numeric value... numberOfFiles+1 RMSE 'mean_rmse'
Set numeric value... numberOfFiles+1 Correlation 'mean_corr'
Save as tab-separated file... 'directory$'/accuracy_learning.txt
Remove

select Strings FileList
Remove
printline Target learning completed!
printline mean RMSE = 'mean_rmse'		mean Correlation = 'mean_corr'
